@@ -1,5 +1,6 @@
 (*
   Copyright (C) 2003  Bardur Arantsson
+  Copyright (C) 2004-2010 The Savonet Team
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -16,6 +17,8 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+(** OCaml interface to the ao library. *)
+
 (** Device type. *)
 type t
 
@@ -26,7 +29,24 @@ type driver_kind_t = [ `LIVE | `FILE | `UNKNOWN ]
 type byte_format_t = [ `LITTLE_ENDIAN | `BIG_ENDIAN | `NATIVE | `UNKNOWN ]
 
 (** Driver type (private). *)
-type driver_t
+type driver_t = private
+  { id         : int ;
+    kind       : driver_kind_t ;
+    short_name : string ;
+    name       : string ;
+    comment    : string ;
+    author     : string ;
+    priority   : int ;
+    preferred_byte_format : byte_format_t ;
+    options    : string list
+  }
+
+(** Raised when trying to play or close 
+  * a closed device. *)
+exception Closed
+
+(** Raised when passing an invalid parameter's value. *)
+exception Invalid_value
 
 (** Get default driver *)
 val get_default_driver : unit -> driver_t
@@ -34,44 +54,26 @@ val get_default_driver : unit -> driver_t
 (** [drivers] is a list of available drivers. *)
 val drivers : driver_t list
 
-(** [driver_kind driver] gets the driver type of the given driver. *)
-val driver_kind : driver_t -> driver_kind_t
-
-(** [driver_name driver] gets the name of the given driver. *)
-val driver_name : driver_t -> string
-
-(** [driver_short_name driver] gets the short name of the given driver. *)
-val driver_short_name : driver_t -> string
-
-(** [driver_comment driver] gets the comment associated with the given driver. *)
-val driver_comment : driver_t -> string
-
-(** [driver_author driver] gets the author associated with the given driver. *)
-val driver_author : driver_t -> string
-
-(** [driver_priority driver] gets the priority associated with the given driver. *)
-val driver_priority : driver_t -> int
-
-(** [driver_preferred_byte_format driver] retrieves the preferred byte format of
-  * the given driver. *)
-val driver_preferred_byte_format : driver_t -> byte_format_t
-
-(** [driver_options driver] retrieves the list of supported options
-  for the given driver. *)
-val driver_options : driver_t -> string list
-
-(** [open_live TODO]. *)
+(** [open_live].
+  * The [channels_matrix] parameter is 
+  * used only if the module is compiled against
+  * libao >= 1.0. *)
 val open_live : ?bits:int ->
    ?rate:int ->
    ?channels:int ->
+   ?channels_matrix:string ->
    ?byte_format:byte_format_t ->
    ?options:((string*string) list) ->
    ?driver:driver_t -> unit -> t
 
-(** [open_file TODO]. *)
+(** [open_file]. 
+  * The [channels_matrix] parameter is  
+  * used only if the module is compiled against 
+  * libao >= 1.0. *)
 val open_file : ?bits:int ->
    ?rate:int ->
    ?channels:int ->
+   ?channels_matrix:string ->
    ?byte_format:byte_format_t ->
    ?options:((string*string) list) ->
    ?driver:driver_t ->
@@ -87,3 +89,15 @@ val play : t -> string -> unit
 
 (** [close device] closes the given device. *)
 val close : t -> unit
+
+(** Backward compatibility functions, do not use them in new code. *)
+
+val driver_kind : driver_t -> driver_kind_t
+val driver_name : driver_t -> string
+val driver_short_name : driver_t -> string
+val driver_comment : driver_t -> string
+val driver_author : driver_t -> string
+val driver_priority : driver_t -> int
+val driver_preferred_byte_format : driver_t -> byte_format_t
+val driver_options : driver_t -> string list
+
